package org.jeecg.config;

import org.apache.shiro.mgt.DefaultSessionStorageEvaluator;
import org.apache.shiro.mgt.DefaultSubjectDAO;
import org.apache.shiro.mgt.SecurityManager;
import org.apache.shiro.spring.LifecycleBeanPostProcessor;
import org.apache.shiro.spring.security.interceptor.AuthorizationAttributeSourceAdvisor;
import org.apache.shiro.spring.web.ShiroFilterFactoryBean;
import org.apache.shiro.web.mgt.DefaultWebSecurityManager;
import org.crazycake.shiro.RedisCacheManager;
import org.crazycake.shiro.RedisManager;
import org.jeecg.common.util.oConvertUtils;
import org.jeecg.modules.shiro.authc.ShiroRealm;
import org.jeecg.modules.shiro.authc.aop.JwtFilter;
import org.springframework.aop.framework.autoproxy.DefaultAdvisorAutoProxyCreator;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.context.annotation.Bean;
import org.springframework.context.annotation.Configuration;
import org.springframework.context.annotation.DependsOn;
import org.springframework.util.StringUtils;

import javax.servlet.Filter;
import java.util.HashMap;
import java.util.LinkedHashMap;
import java.util.Map;

/**
 * @author: Scott
 * @date: 2018/2/7
 * @description: shiro 配置类
 */

@Configuration
public class ShiroConfig {
    @Value("${spring.redis.port}")
    private String port;

    @Value("${spring.redis.host}")
    private String host;

    @Value("${spring.redis.password}")
    private String redisPassword;

    /**
     * Filter Chain定义说明
     * <p>
     * 1、一个URL可以配置多个Filter，使用逗号分隔
     * 2、当设置多个过滤器时，全部验证通过，才视为通过
     * 3、部分过滤器可指定参数，如perms，roles
     */
    @Bean("shiroFilter")
    public ShiroFilterFactoryBean shiroFilter(SecurityManager securityManager) {
        ShiroFilterFactoryBean shiroFilterFactoryBean = new ShiroFilterFactoryBean();
        shiroFilterFactoryBean.setSecurityManager(securityManager);
        // 拦截器
        Map<String, String> filterChainDefinitionMap = new LinkedHashMap<>();
        // 配置不会被拦截的链接 顺序判断
        //登录接口排除
        filterChainDefinitionMap.put("/sys/mLogin", "anon");
        filterChainDefinitionMap.put("/sys/login", "anon");
        filterChainDefinitionMap.put("/sys/wxlogin", "anon");
        filterChainDefinitionMap.put("/ctop/rule/**", "anon");
        //第三方登录
        filterChainDefinitionMap.put("/thirdLogin/**", "anon");
        //获取加密串
        filterChainDefinitionMap.put("/sys/getEncryptedString", "anon");
        //短信验证码
        filterChainDefinitionMap.put("/sys/sms", "anon");
        //手机登录
        filterChainDefinitionMap.put("/sys/phoneLogin", "anon");
        //校验用户是否存在
        filterChainDefinitionMap.put("/sys/user/checkOnlyUser", "anon");
        //用户注册
        filterChainDefinitionMap.put("/sys/user/register", "anon");
        //根据手机号获取用户信息
        filterChainDefinitionMap.put("/sys/user/querySysUser", "anon");
        //用户忘记密码验证手机号
        filterChainDefinitionMap.put("/sys/user/phoneVerification", "anon");
        //用户更改密码
        filterChainDefinitionMap.put("/sys/user/passwordChange", "anon");
        //登录验证码
        filterChainDefinitionMap.put("/auth/2step-code", "anon");
        //图片预览不限制token
        filterChainDefinitionMap.put("/sys/common/view/**", "anon");
        //文件下载不限制token
        filterChainDefinitionMap.put("/sys/common/download/**", "anon");
        //pdf预览
        filterChainDefinitionMap.put("/sys/common/pdf/**", "anon");
        filterChainDefinitionMap.put("/ctop/bytedanceAreaInfo/**", "anon");
        filterChainDefinitionMap.put("/ctop/bytedanceInterestCategory/**", "anon");
        filterChainDefinitionMap.put("/template/creative/words/get", "anon");
        filterChainDefinitionMap.put("template/bytedance/industry/list", "anon");
        //积木报表排除
        filterChainDefinitionMap.put("/jmreport/**", "anon");
        filterChainDefinitionMap.put("/**/*.js.map", "anon");
        filterChainDefinitionMap.put("/**/*.css.map", "anon");
        filterChainDefinitionMap.put("/bytedance/batch/creative/words/get", "anon");
        filterChainDefinitionMap.put("/bytedance/batch/action/text", "anon");
        filterChainDefinitionMap.put("/bytedance/batch/industry/list", "anon");
        filterChainDefinitionMap.put("/appium/crawler/*", "anon");

        filterChainDefinitionMap.put("/ctop/kuaishouChannelRegInfo/*", "anon");
        filterChainDefinitionMap.put("/ctop/byteDance/homePage/report/*", "anon");
        filterChainDefinitionMap.put("/ctop/material/top/cost", "anon");

        filterChainDefinitionMap.put("/v1/**", "anon");

        filterChainDefinitionMap.put("/generic/**", "anon");
        filterChainDefinitionMap.put("/", "anon");
        filterChainDefinitionMap.put("/doc.html", "anon");
        filterChainDefinitionMap.put("/**/*.js", "anon");
        filterChainDefinitionMap.put("/**/*.css", "anon");
        filterChainDefinitionMap.put("/**/*.html", "anon");
        filterChainDefinitionMap.put("/**/*.svg", "anon");
        filterChainDefinitionMap.put("/**/*.jpg", "anon");
        filterChainDefinitionMap.put("/**/*.png", "anon");
        filterChainDefinitionMap.put("/**/*.ico", "anon");
        filterChainDefinitionMap.put("/druid/**", "anon");
        filterChainDefinitionMap.put("/swagger-ui.html", "anon");
        filterChainDefinitionMap.put("/swagger**/**", "anon");
        filterChainDefinitionMap.put("/webjars/**", "anon");
        filterChainDefinitionMap.put("/v2/**", "anon");
        filterChainDefinitionMap.put("/kuaishou", "anon");
        filterChainDefinitionMap.put("/kuaishou/report", "anon");
        filterChainDefinitionMap.put("/ctop/bindAccount/**", "anon");
        filterChainDefinitionMap.put("/**/*.mp4", "anon");
        //activiti
        filterChainDefinitionMap.put("/activiti/**", "anon");
        filterChainDefinitionMap.put("/diagram-viewer/**", "anon");
        filterChainDefinitionMap.put("/editor-app/**", "anon");
        //大屏设计器排除
        filterChainDefinitionMap.put("/big/screen/**", "anon");

        //性能监控
        filterChainDefinitionMap.put("/actuator/metrics/**", "anon");
        filterChainDefinitionMap.put("/actuator/httptrace/**", "anon");
        filterChainDefinitionMap.put("/actuator/redis/**", "anon");
        //消息队列
        filterChainDefinitionMap.put("/mq/**", "anon");
        //模板测试
        filterChainDefinitionMap.put("/test/**", "anon");
        //oauth接口
        filterChainDefinitionMap.put("/auth/**", "anon");
        filterChainDefinitionMap.put("/ctop/syncdata/**", "anon");
        filterChainDefinitionMap.put("/sys/dictItem/list", "anon");
        //头条接口
        filterChainDefinitionMap.put("/bytedance", "anon");
        filterChainDefinitionMap.put("/toutiao/loadAllData", "anon");
        filterChainDefinitionMap.put("/toutiao/loadSingleData", "anon");
        filterChainDefinitionMap.put("/kuaishou/getReport/historyData", "anon");
        filterChainDefinitionMap.put("/toutiao/advertiser/**", "anon");
        filterChainDefinitionMap.put("/template/convert/**", "anon");
        filterChainDefinitionMap.put("/upload/**", "anon");
        filterChainDefinitionMap.put("/appium/appiumTask/run", "anon");

        filterChainDefinitionMap.put("/system/userCompany/**", "anon");
        filterChainDefinitionMap.put("/qywexin", "anon");
        filterChainDefinitionMap.put("/sys/feishu/**", "anon");
        filterChainDefinitionMap.put("/test/getGroupData/**", "anon");
        filterChainDefinitionMap.put("/test/getVideoAndImage/**", "anon");
        filterChainDefinitionMap.put("/test/gerCreative/**", "anon");
        filterChainDefinitionMap.put("/test/testdouyinmusic", "anon");
        filterChainDefinitionMap.put("/test/syncDouyinMusic", "anon");
        filterChainDefinitionMap.put("/test/parseDouyinMusic", "anon");
        filterChainDefinitionMap.put("/report/kuaishouReportDailyAgent/get", "anon");

        filterChainDefinitionMap.put("/appium/appiumTask/run/login", "anon");
        filterChainDefinitionMap.put("/zip/download", "anon");
        //WebSocket
        filterChainDefinitionMap.put("/websocket/*", "anon");
        //爬虫接口
        filterChainDefinitionMap.put("/graphql/video", "anon");
        filterChainDefinitionMap.put("/ks/web/test", "anon");
        filterChainDefinitionMap.put("/ureport/**", "anon");
        filterChainDefinitionMap.put("/job/test/*", "anon");
        filterChainDefinitionMap.put("/third/*", "anon");
        filterChainDefinitionMap.put("/ctop/kuaishou/dailyPage/**", "anon");
        filterChainDefinitionMap.put("/rest/script/*", "anon");
        filterChainDefinitionMap.put("/ctop/checkTaskList/*", "anon");
        filterChainDefinitionMap.put("/ctop/report/bytedance/*", "anon");
        filterChainDefinitionMap.put("/shiwan/shiwanFileUpload/*", "anon");
        filterChainDefinitionMap.put("/oa/wechatNoList/*", "anon");
        filterChainDefinitionMap.put("/ruleGroup/*", "anon");
        filterChainDefinitionMap.put("/kuaishou/material/*", "anon");
        filterChainDefinitionMap.put("/kuaishou/create/*", "anon");
        filterChainDefinitionMap.put("/template/kuaiShouBatchCreativeTemplate/*", "anon");
        filterChainDefinitionMap.put("/template/kuaiShouBatchGroupTemplate/*", "anon");
        filterChainDefinitionMap.put("/template/kuaiShouBatchCampaignTemplate/*", "anon");
        filterChainDefinitionMap.put("/template/**", "anon");
        filterChainDefinitionMap.put("/batch/kuaiShouGroupTemplate/*", "anon");
        filterChainDefinitionMap.put("/kuaishouBatch/**", "anon");
        filterChainDefinitionMap.put("/kuaishou/creative/*", "anon");
        filterChainDefinitionMap.put("/ai/aiKuaiShouAppInfo/*", "anon");
        filterChainDefinitionMap.put("/explore/*", "anon");
        filterChainDefinitionMap.put("/ai/batchUpdate/*", "anon");
        filterChainDefinitionMap.put("/ctop/kuaishou/videoReport/*", "anon");
        //filterChainDefinitionMap.put("/overView/*", "anon");

        // 添加自己的过滤器并且取名为jwt
        Map<String, Filter> filterMap = new HashMap<>(1);
        filterMap.put("jwt", new JwtFilter());
        shiroFilterFactoryBean.setFilters(filterMap);
        // <!-- 过滤链定义，从上向下顺序执行，一般将/**放在最为下边
        filterChainDefinitionMap.put("/**", "jwt");

        // 未授权界面返回JSON
        shiroFilterFactoryBean.setUnauthorizedUrl("/sys/common/403");
        shiroFilterFactoryBean.setLoginUrl("/sys/common/403");
        shiroFilterFactoryBean.setFilterChainDefinitionMap(filterChainDefinitionMap);
        return shiroFilterFactoryBean;
    }

    @Bean("securityManager")
    public DefaultWebSecurityManager securityManager(ShiroRealm myRealm) {
        DefaultWebSecurityManager securityManager = new DefaultWebSecurityManager();
        securityManager.setRealm(myRealm);

        /*
         * 关闭shiro自带的session，详情见文档
         * http://shiro.apache.org/session-management.html#SessionManagement-
         * StatelessApplications%28Sessionless%29
         */
        DefaultSubjectDAO subjectDAO = new DefaultSubjectDAO();
        DefaultSessionStorageEvaluator defaultSessionStorageEvaluator = new DefaultSessionStorageEvaluator();
        defaultSessionStorageEvaluator.setSessionStorageEnabled(false);
        subjectDAO.setSessionStorageEvaluator(defaultSessionStorageEvaluator);
        securityManager.setSubjectDAO(subjectDAO);

        return securityManager;
    }

    /**
     * 下面的代码是添加注解支持
     *
     * @return
     */
    @Bean
    @DependsOn("lifecycleBeanPostProcessor")
    public DefaultAdvisorAutoProxyCreator defaultAdvisorAutoProxyCreator() {
        DefaultAdvisorAutoProxyCreator defaultAdvisorAutoProxyCreator = new DefaultAdvisorAutoProxyCreator();
        defaultAdvisorAutoProxyCreator.setProxyTargetClass(true);
        /**
         * 解决重复代理问题 github#994
         * 添加前缀判断 不匹配 任何Advisor
         */
        defaultAdvisorAutoProxyCreator.setUsePrefix(true);
        defaultAdvisorAutoProxyCreator.setAdvisorBeanNamePrefix("_no_advisor");
        return defaultAdvisorAutoProxyCreator;
    }

    @Bean
    public static LifecycleBeanPostProcessor lifecycleBeanPostProcessor() {
        return new LifecycleBeanPostProcessor();
    }

    @Bean
    public AuthorizationAttributeSourceAdvisor authorizationAttributeSourceAdvisor(DefaultWebSecurityManager securityManager) {
        AuthorizationAttributeSourceAdvisor advisor = new AuthorizationAttributeSourceAdvisor();
        advisor.setSecurityManager(securityManager);
        return advisor;
    }

    /**
     * cacheManager 缓存 redis实现
     * 使用的是shiro-redis开源插件
     *
     * @return
     */
    public RedisCacheManager redisCacheManager() {
        RedisCacheManager redisCacheManager = new RedisCacheManager();
        redisCacheManager.setRedisManager(redisManager());
        //redis中针对不同用户缓存(此处的id需要对应user实体中的id字段,用于唯一标识)
        redisCacheManager.setPrincipalIdFieldName("id");
        //用户权限信息缓存时间
        redisCacheManager.setExpire(200000);
        return redisCacheManager;
    }

    /**
     * 配置shiro redisManager
     * 使用的是shiro-redis开源插件
     *
     * @return
     */
    @Bean
    public RedisManager redisManager() {
        RedisManager redisManager = new RedisManager();
        redisManager.setHost(host);
        redisManager.setPort(oConvertUtils.getInt(port));
        redisManager.setTimeout(0);
        if (!StringUtils.isEmpty(redisPassword)) {
            redisManager.setPassword(redisPassword);
        }
        return redisManager;
    }

}
