package cn.com.ctop.job.kuaishou.handler;

import cn.com.ctop.kuaishou.modules.ai.entity.AiKuaishouAdvertiserStrategy;
import cn.com.ctop.kuaishou.modules.ai.service.IAiKuaishouAdvertiserStrategyService;
import cn.com.ctop.kuaishou.modules.ai.service.IAiKuaishouCreateCreativeService;
import com.xxl.job.core.handler.annotation.XxlJob;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Component;

import java.text.ParseException;
import java.util.List;

/**
 * @author syh
 * 自动投放定时任务
 */
@Component
public class KuaishouAiCreativeJob {
    @Autowired
    private IAiKuaishouAdvertiserStrategyService strategyService;
    @Autowired
    private IAiKuaishouCreateCreativeService createCreativeService;
    /**
     * 自定义上新创意
     */
    @XxlJob("kuaishouCustomCreativeAuto")
    public void kuaishouCustomCreativeAuto() {
        List<AiKuaishouAdvertiserStrategy> strategies = strategyService.getAllEffectStrategy();
        if(null==strategies||strategies.isEmpty()){
            return;
        }
        strategies.forEach(strategy -> {
            if(null!=strategy.getOpenProgramCreate()&&strategy.getOpenProgramCreate()!=1){
                createCreativeService.autoCreateCreative(strategy,1,null);
            }
        });
    }

    /**
     * 补充自定义创意
     */
    @XxlJob("kuaishouCustomCreativeSupplement")
    public void kuaishouCustomCreativeSupplement() {
        List<AiKuaishouAdvertiserStrategy> strategies = strategyService.getAllEffectStrategy();
        if(null==strategies||strategies.isEmpty()){
            return;
        }
        strategies.forEach(strategy -> {
            if(null!=strategy.getOpenProgramCreate()&&strategy.getOpenProgramCreate()!=1){
                try {
                    createCreativeService.customCreativeSupplement(strategy,1);
                } catch (ParseException e) {
                    e.printStackTrace();
                }
            }
        });
    }


    /**
     * 自定义高质量创意
     */
    @XxlJob("kuaishouCustomCreativeTop")
    public void kuaishouCustomCreativeTop() {
        List<AiKuaishouAdvertiserStrategy> strategies = strategyService.getAllEffectStrategy();
        if(null==strategies||strategies.isEmpty()){
            return;
        }
        strategies.forEach(strategy -> {
            if(null!=strategy.getOpenProgramCreate()&&strategy.getOpenProgramCreate()!=1){
                createCreativeService.autoCreateCreative(strategy,2,20L);
            }
        });
    }

    /**
     * 自定义遗漏创意
     */
    @XxlJob("kuaishouCustomCreativeMissing")
    public void kuaishouCustomCreativeMissing() {
        List<AiKuaishouAdvertiserStrategy> strategies = strategyService.getAllEffectStrategy();
        if(null==strategies||strategies.isEmpty()){
            return;
        }
        strategies.forEach(strategy -> {
            if(null!=strategy.getOpenProgramCreate()&&strategy.getOpenProgramCreate()!=1){
                createCreativeService.autoCreateCreative(strategy,3,null);
            }
        });
    }

    /**
     * 自定义历史打捞创意
     */
    @XxlJob("kuaishouCustomCreativeHistory")
    public void kuaishouCustomCreativeHistory() {
        List<AiKuaishouAdvertiserStrategy> strategies = strategyService.getAllEffectStrategy();
        if(null==strategies||strategies.isEmpty()){
            return;
        }
        strategies.forEach(strategy -> {
            if(null!=strategy.getOpenProgramCreate()&&strategy.getOpenProgramCreate()!=1){
                createCreativeService.autoCreateCreative(strategy,4,null);
            }
        });
    }

    /**
     * 程序化上新创意
     */
    @XxlJob("kuaishouProgramCreativeAuto")
    public void kuaishouProgramCreativeAuto() {
        List<AiKuaishouAdvertiserStrategy> strategies = strategyService.getAllEffectStrategy();
        if(null==strategies||strategies.isEmpty()){
            return;
        }
        strategies.forEach(strategy -> {
            if(null!=strategy.getOpenProgramCreate()&&strategy.getOpenProgramCreate()!=0){
                createCreativeService.autoCreateProgramCreative(strategy,1,150L);
            }
        });
    }

    /**
     * 程序化上新创意
     */
    @XxlJob("kuaishouProgramCreativeTop")
    public void kuaishouProgramCreativeTop() {
        List<AiKuaishouAdvertiserStrategy> strategies = strategyService.getAllEffectStrategy();
        if(null==strategies||strategies.isEmpty()){
            return;
        }
        strategies.forEach(strategy -> {
            if(null!=strategy.getOpenProgramCreate()&&strategy.getOpenProgramCreate()!=0){
                createCreativeService.autoCreateProgramCreative(strategy,2,150L);
            }
        });
    }
}
