package cn.com.ctop.kuaishou.modules.batch.controller;

import cn.com.ctop.common.module.annotation.AutoLog;
import cn.com.ctop.common.module.entity.CtopOauthToken;
import cn.com.ctop.common.module.service.ICtopOauthTokenService;
import cn.com.ctop.common.module.utils.Check;
import cn.com.ctop.common.module.utils.QueryGenerator;
import cn.com.ctop.kuaishou.modules.batch.entity.KuaishouBatchCampaignPreview;
import cn.com.ctop.kuaishou.modules.batch.service.IKuaiShouUpdateService;
import cn.com.ctop.kuaishou.modules.batch.service.IKuaishouBatchCampaignPreviewService;
import com.alibaba.fastjson.JSON;
import com.alibaba.fastjson.JSONObject;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import lombok.extern.slf4j.Slf4j;
import org.jeecg.common.api.vo.Result;
import org.jeecg.common.util.oConvertUtils;
import org.jeecgframework.poi.excel.ExcelImportUtil;
import org.jeecgframework.poi.excel.def.NormalExcelConstants;
import org.jeecgframework.poi.excel.entity.ExportParams;
import org.jeecgframework.poi.excel.entity.ImportParams;
import org.jeecgframework.poi.excel.view.JeecgEntityExcelView;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RequestMethod;
import org.springframework.web.bind.annotation.RequestParam;
import org.springframework.web.bind.annotation.RestController;
import org.springframework.web.multipart.MultipartFile;
import org.springframework.web.multipart.MultipartHttpServletRequest;
import org.springframework.web.servlet.ModelAndView;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import java.io.IOException;
import java.io.UnsupportedEncodingException;
import java.net.URLDecoder;
import java.util.Arrays;
import java.util.List;
import java.util.Map;

/**
 * 快手-广告计划预览信息
 *
 * @author jeecg-boot
 * @version V1.0
 * @date 2021-01-26
 */
@Slf4j
@Api(tags = "快手-广告计划预览信息")
@RestController
@RequestMapping("/kuaishouBatch/campaignPreview")
public class KuaishouBatchCampaignPreviewController {
    @Autowired
    private IKuaishouBatchCampaignPreviewService kuaishouBatchCampaignPreviewService;
    @Autowired
    private IKuaishouBatchCampaignPreviewService campaignPreviewService;
    @Autowired
    private ICtopOauthTokenService tokenService;
    @Autowired
    private IKuaiShouUpdateService kuaiShouUpdateService;

    /**
     * 分页列表查询
     *
     * @param kuaishouBatchCampaignPreview
     * @param pageNo
     * @param pageSize
     * @param req
     * @return
     */
    @AutoLog(value = "快手-广告计划预览信息-分页列表查询")
    @ApiOperation(value = "快手-广告计划预览信息-分页列表查询", notes = "快手-广告计划预览信息-分页列表查询")
    @GetMapping(value = "/list")
    public Result<IPage<KuaishouBatchCampaignPreview>> queryPageList(KuaishouBatchCampaignPreview kuaishouBatchCampaignPreview,
                                                                     @RequestParam(name = "pageNo", defaultValue = "1") Integer pageNo,
                                                                     @RequestParam(name = "pageSize", defaultValue = "10") Integer pageSize,
                                                                     HttpServletRequest req) {
        Result<IPage<KuaishouBatchCampaignPreview>> result = new Result<>();
        String statDate = kuaishouBatchCampaignPreview.getStatDate();
        kuaishouBatchCampaignPreview.setStatDate(null);
        QueryWrapper<KuaishouBatchCampaignPreview> queryWrapper = QueryGenerator.initQueryWrapper(kuaishouBatchCampaignPreview, req.getParameterMap());
        if (!Check.isNull(statDate)) {
            queryWrapper.like("create_time", statDate);
        }
        queryWrapper.orderByDesc("create_time");
        Page<KuaishouBatchCampaignPreview> page = new Page<KuaishouBatchCampaignPreview>(pageNo, pageSize);
        IPage<KuaishouBatchCampaignPreview> pageList = kuaishouBatchCampaignPreviewService.page(page, queryWrapper);
        result.setSuccess(true);
        result.setResult(pageList);
        return result;
    }

    /**
     * 添加
     *
     * @param kuaishouBatchCampaignPreview
     * @return
     */
    @AutoLog(value = "快手-广告计划预览信息-添加")
    @ApiOperation(value = "快手-广告计划预览信息-添加", notes = "快手-广告计划预览信息-添加")
    @PostMapping(value = "/add")
    public Result<KuaishouBatchCampaignPreview> add(@RequestBody KuaishouBatchCampaignPreview kuaishouBatchCampaignPreview) {
        Result<KuaishouBatchCampaignPreview> result = new Result<>();
        try {
            kuaishouBatchCampaignPreviewService.save(kuaishouBatchCampaignPreview);
            result.success("添加成功！");
        } catch (Exception e) {
            log.error(e.getMessage(), e);
            result.error500("操作失败");
        }
        return result;
    }

    /**
     * 添加
     *
     * @param data
     * @return
     */
    @AutoLog(value = "添加预览数据")
    @ApiOperation(value = "添加预览数据", notes = "添加预览数据")
    @PostMapping(value = "/addPreviewData")
    public Result<Object> addPreviewData(@RequestBody JSONObject data) {
        Result<Object> result = new Result<>();
        try {
            kuaishouBatchCampaignPreviewService.addPreviewData(data);
            result.success("添加成功！");
        } catch (Exception e) {
            log.error(e.getMessage(), e);
            result.error500("操作失败，" + e.getMessage());
        }
        return result;
    }

    @AutoLog(value = "通过账户和批次标识查询预览数据")
    @ApiOperation(value = "通过账户和批次标识查询预览数据", notes = "通过账户和批次标识查询预览数据")
    @GetMapping(value = "/queryByBatchId")
    public Result<Object> queryByBatchId(String batchId, String accountId) {
        try {
            if (Check.isNull(batchId)) {
                return Result.error("批次号为空");
            }
            if (Check.isNull(accountId)) {
                return Result.error("请选择账户");
            }
            JSONObject obj = kuaishouBatchCampaignPreviewService.queryByBatchId(batchId, accountId);
            return Result.ok(obj);
        } catch (Exception e) {
            log.error("queryByBatchId is error", e);
            return Result.error("查询异常");
        }
    }

    /**
     * 编辑
     *
     * @param kuaishouBatchCampaignPreview
     * @return
     */
    @AutoLog(value = "快手-广告计划预览信息-编辑")
    @ApiOperation(value = "快手-广告计划预览信息-编辑", notes = "快手-广告计划预览信息-编辑")
    @PostMapping(value = "/edit")
    public Result<KuaishouBatchCampaignPreview> edit(@RequestBody KuaishouBatchCampaignPreview kuaishouBatchCampaignPreview) {
        Result<KuaishouBatchCampaignPreview> result = new Result<KuaishouBatchCampaignPreview>();
        KuaishouBatchCampaignPreview kuaishouBatchCampaignPreviewEntity = kuaishouBatchCampaignPreviewService.getById(kuaishouBatchCampaignPreview.getId());
        if (kuaishouBatchCampaignPreviewEntity == null) {
            result.error500("未找到对应实体");
        } else {
            boolean ok = kuaishouBatchCampaignPreviewService.updateById(kuaishouBatchCampaignPreview);
            if (ok) {
                result.success("修改成功!");
            }
        }

        return result;
    }

    /**
     * 手动修改定时启动
     *
     * @param kuaishouBatchCampaignPreview
     * @return
     */
    @AutoLog(value = "手动修改定时启动")
    @ApiOperation(value = "手动修改定时启动", notes = "手动修改定时启动")
    @PostMapping(value = "/updateStatus")
    public Result<Object> kuaishouBatchCampaignTimeStartJob() {
        try {
            List<KuaishouBatchCampaignPreview> campaignPreviewList = campaignPreviewService.queryListByTime();
            if (!Check.isNull(campaignPreviewList)) {
                for (KuaishouBatchCampaignPreview campaignPreview : campaignPreviewList) {
                    CtopOauthToken token = tokenService.getTokenByAccountId(campaignPreview.getAccountId());
                    if (!Check.isNull(token)) {
                        Map<String, Object> map = kuaiShouUpdateService.updateCampaignStatus(token.getAccessToken(), campaignPreview.getAccountId(), campaignPreview.getCampaignId(), 1, "timingStart");
                        if ((boolean) map.get("success")) {
                            QueryWrapper<KuaishouBatchCampaignPreview> queryWrapper = new QueryWrapper<>();
                            queryWrapper.eq("id", campaignPreview.getId());
                            KuaishouBatchCampaignPreview campaign = new KuaishouBatchCampaignPreview();
                            campaign.setMessage("定时启动成功！");
                            campaignPreviewService.update(campaign, queryWrapper);
                        }
                    }
                }
            }
        } catch (Exception e) {
            return Result.error("error:" + e.getMessage());
        }
        return Result.ok("success");
    }

    /**
     * 通过id删除
     *
     * @param id
     * @return
     */
    @AutoLog(value = "快手-广告计划预览信息-通过id删除")
    @ApiOperation(value = "快手-广告计划预览信息-通过id删除", notes = "快手-广告计划预览信息-通过id删除")
    @RequestMapping(value = "/delete")
    public Result<?> delete(@RequestParam(name = "id", required = true) String id) {
        try {
            kuaishouBatchCampaignPreviewService.removeById(id);
        } catch (Exception e) {
            log.error("删除失败", e.getMessage());
            return Result.error("删除失败!");
        }
        return Result.ok("删除成功!");
    }

    /**
     * 批量删除
     *
     * @param ids
     * @return
     */
    @AutoLog(value = "快手-广告计划预览信息-批量删除")
    @ApiOperation(value = "快手-广告计划预览信息-批量删除", notes = "快手-广告计划预览信息-批量删除")
    @RequestMapping(value = "/deleteBatch")
    public Result<KuaishouBatchCampaignPreview> deleteBatch(@RequestParam(name = "ids", required = true) String ids) {
        Result<KuaishouBatchCampaignPreview> result = new Result<>();
        if (ids == null || "".equals(ids.trim())) {
            result.error500("参数不识别！");
        } else {
            this.kuaishouBatchCampaignPreviewService.removeByIds(Arrays.asList(ids.split(",")));
            result.success("删除成功!");
        }
        return result;
    }

    /**
     * 通过id查询
     *
     * @param id
     * @return
     */
    @AutoLog(value = "快手-广告计划预览信息-通过id查询")
    @ApiOperation(value = "快手-广告计划预览信息-通过id查询", notes = "快手-广告计划预览信息-通过id查询")
    @GetMapping(value = "/queryById")
    public Result<KuaishouBatchCampaignPreview> queryById(@RequestParam(name = "id", required = true) String id) {
        Result<KuaishouBatchCampaignPreview> result = new Result<>();
        KuaishouBatchCampaignPreview kuaishouBatchCampaignPreview = kuaishouBatchCampaignPreviewService.getById(id);
        if (kuaishouBatchCampaignPreview == null) {
            result.error500("未找到对应实体");
        } else {
            result.setResult(kuaishouBatchCampaignPreview);
            result.setSuccess(true);
        }
        return result;
    }

    /**
     * 导出excel
     *
     * @param request
     * @param response
     */
    @RequestMapping(value = "/exportXls")
    public ModelAndView exportXls(HttpServletRequest request, HttpServletResponse response) {
        // Step.1 组装查询条件
        QueryWrapper<KuaishouBatchCampaignPreview> queryWrapper = null;
        try {
            String paramsStr = request.getParameter("paramsStr");
            if (oConvertUtils.isNotEmpty(paramsStr)) {
                String deString = URLDecoder.decode(paramsStr, "UTF-8");
                KuaishouBatchCampaignPreview kuaishouBatchCampaignPreview = JSON.parseObject(deString, KuaishouBatchCampaignPreview.class);
                queryWrapper = QueryGenerator.initQueryWrapper(kuaishouBatchCampaignPreview, request.getParameterMap());
            }
        } catch (UnsupportedEncodingException e) {
            e.printStackTrace();
        }

        //Step.2 AutoPoi 导出Excel
        ModelAndView mv = new ModelAndView(new JeecgEntityExcelView());
        List<KuaishouBatchCampaignPreview> pageList = kuaishouBatchCampaignPreviewService.list(queryWrapper);
        //导出文件名称
        mv.addObject(NormalExcelConstants.FILE_NAME, "快手-广告计划预览信息列表");
        mv.addObject(NormalExcelConstants.CLASS, KuaishouBatchCampaignPreview.class);
        mv.addObject(NormalExcelConstants.PARAMS, new ExportParams("快手-广告计划预览信息列表数据", "导出人:Jeecg", "导出信息"));
        mv.addObject(NormalExcelConstants.DATA_LIST, pageList);
        return mv;
    }

    /**
     * 通过excel导入数据
     *
     * @param request
     * @param response
     * @return
     */
    @RequestMapping(value = "/importExcel", method = RequestMethod.POST)
    public Result<?> importExcel(HttpServletRequest request, HttpServletResponse response) {
        MultipartHttpServletRequest multipartRequest = (MultipartHttpServletRequest) request;
        Map<String, MultipartFile> fileMap = multipartRequest.getFileMap();
        for (Map.Entry<String, MultipartFile> entity : fileMap.entrySet()) {
            MultipartFile file = entity.getValue();
            ImportParams params = new ImportParams();
            params.setTitleRows(2);
            params.setHeadRows(1);
            params.setNeedSave(true);
            try {
                List<KuaishouBatchCampaignPreview> listKuaishouBatchCampaignPreviews = ExcelImportUtil.importExcel(file.getInputStream(), KuaishouBatchCampaignPreview.class, params);
                kuaishouBatchCampaignPreviewService.saveBatch(listKuaishouBatchCampaignPreviews);
                return Result.ok("文件导入成功！数据行数:" + listKuaishouBatchCampaignPreviews.size());
            } catch (Exception e) {
                log.error(e.getMessage(), e);
                return Result.error("文件导入失败:" + e.getMessage());
            } finally {
                try {
                    file.getInputStream().close();
                } catch (IOException e) {
                    e.printStackTrace();
                }
            }
        }
        return Result.ok("文件导入失败！");
    }

}
