package cn.com.ctop.kuaishou.modules.report.controller;

import cn.com.ctop.common.module.utils.QueryGenerator;
import cn.com.ctop.kuaishou.modules.report.entity.KuaishouGroupExplore;
import cn.com.ctop.kuaishou.modules.report.service.IKuaishouGroupExploreService;
import com.alibaba.fastjson.JSONObject;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import io.swagger.annotations.Api;
import lombok.extern.slf4j.Slf4j;
import org.jeecg.common.api.vo.Result;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.*;

import javax.servlet.http.HttpServletRequest;
import java.util.Arrays;

/**
 * 广告组的加速探索报表
 *
 * @author jeecg-boot
 * @version V1.0
 * @date 2021-02-02
 */
@Slf4j
@Api(tags = "广告组的加速探索报表")
@RestController
@RequestMapping("/explore")
public class KuaishouGroupExploreController {
    @Autowired
    private IKuaishouGroupExploreService kuaishouGroupExploreService;

    /**
     * 分页列表查询
     *
     * @param kuaishouGroupExplore
     * @param pageNo
     * @param pageSize
     * @param req
     * @return
     */
    @GetMapping(value = "/list")
    public Result<IPage<KuaishouGroupExplore>> queryPageList(KuaishouGroupExplore kuaishouGroupExplore,
                                                             @RequestParam(name = "pageNo", defaultValue = "1") Integer pageNo,
                                                             @RequestParam(name = "pageSize", defaultValue = "10") Integer pageSize,
                                                             HttpServletRequest req) {
        Result<IPage<KuaishouGroupExplore>> result = new Result<>();
        QueryWrapper<KuaishouGroupExplore> queryWrapper = QueryGenerator.initQueryWrapper(kuaishouGroupExplore, req.getParameterMap());
        Page<KuaishouGroupExplore> page = new Page<KuaishouGroupExplore>(pageNo, pageSize);
        IPage<KuaishouGroupExplore> pageList = kuaishouGroupExploreService.page(page, queryWrapper);
        result.setSuccess(true);
        result.setResult(pageList);
        return result;
    }

    /**
     * 查询加速探索信息接口
     *
     * @param data
     * @return
     */
    @PostMapping(value = "/unitExploreInfoGet")
    public Result<Object> unitExploreInfoGet(@RequestBody JSONObject data) {
        try {
            return kuaishouGroupExploreService.unitExploreInfoGet(data);
        } catch (Exception e) {
            log.error(e.getMessage(), e);
            return Result.error("操作失败");
        }
    }

    /**
     * 广告组的加速探索预算设置接口
     *
     * @param data
     * @return
     */
    @PostMapping(value = "/unitExploreBudgetUpdate")
    public Result<Object> unitExploreBudgetUpdate(@RequestBody JSONObject data) {
        try {
            return kuaishouGroupExploreService.unitExploreBudgetUpdate(data);
        } catch (Exception e) {
            log.error(e.getMessage(), e);
            return Result.error("操作失败");
        }
    }

    /**
     * 暂停广告组的加速探索预算接口
     *
     * @param data
     * @return
     */
    @PostMapping(value = "/unitExploreStatusPause")
    public Result<Object> unitExploreStatusPause(@RequestBody JSONObject data) {
        try {
            return kuaishouGroupExploreService.unitExploreStatusPause(data);
        } catch (Exception e) {
            log.error(e.getMessage(), e);
            return Result.error("操作失败");
        }
    }

    /**
     * 查询广告组的加速探索报表接口
     *
     * @param data
     * @return
     */
    @PostMapping(value = "/querySpeedExploreReport")
    public Result<Object> querySpeedExploreReport(@RequestBody JSONObject data) {
        try {
            return kuaishouGroupExploreService.querySpeedExploreReport(data);
        } catch (Exception e) {
            log.error(e.getMessage(), e);
            return Result.error("操作失败");
        }
    }


    /**
     * 添加
     *
     * @param kuaishouGroupExplore
     * @return
     */
    @PostMapping(value = "/add")
    public Result<KuaishouGroupExplore> add(@RequestBody KuaishouGroupExplore kuaishouGroupExplore) {
        Result<KuaishouGroupExplore> result = new Result<>();
        try {
            kuaishouGroupExploreService.save(kuaishouGroupExplore);
            result.success("添加成功！");
        } catch (Exception e) {
            log.error(e.getMessage(), e);
            result.error500("操作失败");
        }
        return result;
    }

    /**
     * 编辑
     *
     * @param kuaishouGroupExplore
     * @return
     */
    @PutMapping(value = "/edit")
    public Result<KuaishouGroupExplore> edit(@RequestBody KuaishouGroupExplore kuaishouGroupExplore) {
        Result<KuaishouGroupExplore> result = new Result<KuaishouGroupExplore>();
        KuaishouGroupExplore kuaishouGroupExploreEntity = kuaishouGroupExploreService.getById(kuaishouGroupExplore.getId());
        if (kuaishouGroupExploreEntity == null) {
            result.error500("未找到对应实体");
        } else {
            boolean ok = kuaishouGroupExploreService.updateById(kuaishouGroupExplore);
            if (ok) {
                result.success("修改成功!");
            }
        }

        return result;
    }

    /**
     * 通过id删除
     *
     * @param id
     * @return
     */
    @DeleteMapping(value = "/delete")
    public Result<?> delete(@RequestParam(name = "id", required = true) String id) {
        try {
            kuaishouGroupExploreService.removeById(id);
        } catch (Exception e) {
            log.error("删除失败", e.getMessage());
            return Result.error("删除失败!");
        }
        return Result.ok("删除成功!");
    }

    /**
     * 批量删除
     *
     * @param ids
     * @return
     */
    @DeleteMapping(value = "/deleteBatch")
    public Result<KuaishouGroupExplore> deleteBatch(@RequestParam(name = "ids", required = true) String ids) {
        Result<KuaishouGroupExplore> result = new Result<>();
        if (ids == null || "".equals(ids.trim())) {
            result.error500("参数不识别！");
        } else {
            this.kuaishouGroupExploreService.removeByIds(Arrays.asList(ids.split(",")));
            result.success("删除成功!");
        }
        return result;
    }

    /**
     * 通过id查询
     *
     * @param id
     * @return
     */
    @GetMapping(value = "/queryById")
    public Result<KuaishouGroupExplore> queryById(@RequestParam(name = "id", required = true) String id) {
        Result<KuaishouGroupExplore> result = new Result<>();
        KuaishouGroupExplore kuaishouGroupExplore = kuaishouGroupExploreService.getById(id);
        if (kuaishouGroupExplore == null) {
            result.error500("未找到对应实体");
        } else {
            result.setResult(kuaishouGroupExplore);
            result.setSuccess(true);
        }
        return result;
    }


}
