package cn.com.ctop.kuaishou.modules.report.service.impl;


import cn.com.ctop.common.module.entity.CtopOauthToken;
import cn.com.ctop.common.module.entity.UserAllocation;
import cn.com.ctop.common.module.mapper.UserAllocationMapper;
import cn.com.ctop.common.module.service.ICtopOauthTokenService;
import cn.com.ctop.common.module.utils.Check;
import cn.com.ctop.kuaishou.modules.batch.mapper.KuaiShouCreativeMapper;
import cn.com.ctop.kuaishou.modules.batch.service.IKuaishouInterfaceService;
import cn.com.ctop.kuaishou.modules.report.entity.EtlKuaishouReportAccountHourly;
import cn.com.ctop.kuaishou.modules.report.enums.CheckParametersEnum;
import cn.com.ctop.kuaishou.modules.report.mapper.EtlKuaishouReportAccountDailyMapper;
import cn.com.ctop.kuaishou.modules.report.mapper.EtlKuaishouReportAccountHourlyMapper;
import cn.com.ctop.kuaishou.modules.report.mapper.KuaishouReportHourlyAccountMapper;
import cn.com.ctop.kuaishou.modules.report.service.IEtlKuaishouReportAccountHourlyService;
import com.alibaba.fastjson.JSONObject;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import org.jeecg.common.api.vo.Result;
import org.jeecg.common.util.DateUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.math.BigDecimal;
import java.math.RoundingMode;
import java.text.ParseException;
import java.text.SimpleDateFormat;
import java.util.ArrayList;
import java.util.Collections;
import java.util.Comparator;
import java.util.Date;
import java.util.List;

/**
 * 账户维度-每小时盯盘数据
 *
 * @author jeecg-boot
 * @version V1.0
 * @date 2021-03-04
 */
@Service
public class EtlKuaishouReportAccountHourlyServiceImpl extends ServiceImpl<EtlKuaishouReportAccountHourlyMapper, EtlKuaishouReportAccountHourly> implements IEtlKuaishouReportAccountHourlyService {

    @Autowired
    private KuaishouReportHourlyAccountMapper hourlyAccountMapper;
    @Autowired
    private KuaiShouCreativeMapper kuaiShouCreativeMapper;
    @Autowired
    private UserAllocationMapper allocationMapper;

    @Autowired
    private EtlKuaishouReportAccountHourlyMapper etlHourlyAccountMapper;

    @Autowired
    private IKuaishouInterfaceService iKuaishouInterfaceService;
    @Autowired
    private EtlKuaishouReportAccountDailyMapper etlDailyAccountMapper;
    @Autowired
    private ICtopOauthTokenService oauthTokenService;

    @Override
    public void queryEveryhourlyDataForAccount() {
        List<EtlKuaishouReportAccountHourly> list = hourlyAccountMapper.queryEveryhourlyDataForAccount();
        if (!Check.isNull(list)) {
            etlHourlyAccountMapper.replaceBatch(list);
        }
        etlDailyAccountMapper.replaceBatch(list);
    }

    @Override
    public Result<Object> getAggregateByHour(JSONObject request) throws ParseException {
        Long accountId = request.getLong("accountId");
        String startTime = request.getString("startTime");
        if (Check.isNull(accountId) || Check.isNull(startTime)) {
            return Result.error("请选择时间和账户");
        }
        String endTime = request.getString("endTime");
        EtlKuaishouReportAccountHourly today = null;
        EtlKuaishouReportAccountHourly yesterday = null;
        if (endTime.equals(startTime)) {
            today = etlHourlyAccountMapper.getOnedayAggregateByHour(startTime, accountId);
            String lastDay = DateUtils.getLastDay(startTime, 1);
            yesterday = etlHourlyAccountMapper.getOnedayAggregateByHour(lastDay, accountId);
        } else {
            today = etlDailyAccountMapper.getOnedayAggregatebyDaily(startTime, endTime, accountId);
            int days = DateUtils.dateDiff(startTime, endTime);
            String endTime2 = DateUtils.getLastDay(startTime, 1);
            String startTime2 = DateUtils.getLastDay(endTime2, days);
            yesterday = etlDailyAccountMapper.getOnedayAggregatebyDaily(startTime2, endTime2, accountId);
        }
        if (!Check.isNull(today) && !Check.isNull(yesterday)) {
            BigDecimal tcharge = today.getCharge();
            BigDecimal ycharge = yesterday.getCharge();
            if (ycharge.compareTo(BigDecimal.ZERO) != 0 && ycharge.compareTo(BigDecimal.ZERO) != 0) {
                BigDecimal grew = tcharge.subtract(ycharge).divide(ycharge, 4, BigDecimal.ROUND_HALF_UP).multiply(new BigDecimal(100)).setScale(2, RoundingMode.HALF_UP);
                today.setGrew(grew);
            }
        }
        JSONObject obj = new JSONObject();
        obj.put("today", today);
        obj.put("yesterday", yesterday);
        return Result.ok(obj);
    }

    @Override
    public Result<Object> getEtlReportList(JSONObject request) {
        Long accountId = request.getLong("accountId");
        String startTime = request.getString("startTime");
        String value = request.getString("value");
        if (Check.isNull(accountId) || Check.isNull(startTime) || Check.isNull(value)) {
            return Result.error("必填参数为空");
        }
        //type alone 是独立字段例如:charge,activation，more是复杂字段
        String type = "more";
        if (CheckParametersEnum.isInclude(value)) {
            type = "alone";
            value = CheckParametersEnum.getSqlName(value);
        }
        String endTime = request.getString("endTime");
        JSONObject obj = new JSONObject();
        List<JSONObject> todayList = null;
        List<JSONObject> yesterdayList = null;
        String updateTime = "";
        if (endTime.equals(startTime)) {
            updateTime = etlHourlyAccountMapper.getNewlyUpdateTime();
            if ("photo_share".equals(value)) {
                value = "share";
            }
            if ("photo_comment".equals(value)) {
                value = "comment";
            }
            if ("play_3s_count".equals(value)) {
                value = value = request.getString("value");
                type = "more";
            }
            todayList = etlHourlyAccountMapper.getOnedayEtlReportList(startTime, accountId, value, type);
            String lastDay = DateUtils.getLastDay(startTime, 1);
            yesterdayList = etlHourlyAccountMapper.getOnedayEtlReportList(lastDay, accountId, value, type);
            completionTime(todayList);
            completionTime(yesterdayList);
        } else {
            if (value.equals("play_3s_ratio")) {
                value = request.getString("value");
                type = "more";
            }
            updateTime = etlDailyAccountMapper.getNewlyUpdateTime();
            todayList = etlDailyAccountMapper.getOnedayEtlReportList(startTime, endTime, accountId, value, type);
            int days = DateUtils.dateDiff(startTime, endTime);
            String endTime2 = DateUtils.getLastDay(startTime, 1);
            String startTime2 = DateUtils.getLastDay(endTime2, days);
            yesterdayList = etlDailyAccountMapper.getOnedayEtlReportList(startTime2, endTime2, accountId, value, type);
            int todayNum = Check.isNull(todayList) ? 0 : todayList.size();
            int yesterdayNum = Check.isNull(yesterdayList) ? 0 : yesterdayList.size();
            days += 1;
            List<String> dayList = null;
            if (days > todayNum) {
                dayList = DateUtils.getDays(startTime, endTime);
                completionDate(todayList, dayList);
            }
            if (days > yesterdayNum) {
                dayList = DateUtils.getDays(startTime2, endTime2);
                completionDate(yesterdayList, dayList);
            }
            statDateListSort(todayList);
            statDateListSort(yesterdayList);
        }
        obj.put("today", todayList);
        obj.put("yesterday", yesterdayList);
        obj.put("updateTime", updateTime);
        return Result.ok(obj);
    }

    /**
     * 补全日期
     */
    private void completionDate(List<JSONObject> dataList, List<String> dayList) {
        List<String> newDayList = new ArrayList<>();
        for (JSONObject data : dataList) {
            newDayList.add(data.getString("statDate"));
        }
        for (String day : dayList) {
            if (!newDayList.contains(day)) {
                JSONObject obj = new JSONObject();
                obj.put("statDate", day);
                obj.put("value", 0);
                dataList.add(obj);
            }
        }
    }

    /**
     * 补全时辰
     */
    private void completionTime(List<JSONObject> dataList) {
        List<Integer> newDayList = new ArrayList<>();
        for (JSONObject data : dataList) {
            newDayList.add(data.getInteger("statHour"));
        }
        List<Integer> dayList = new ArrayList<>();
        for (int i = 0; i < 23; i++) {
            dayList.add(i);
        }
        for (Integer hour : dayList) {
            if (!newDayList.contains(hour)) {
                JSONObject obj = new JSONObject();
                obj.put("statHour", hour);
                obj.put("value", 0);
                dataList.add(obj);
            }
        }
        statHourListSort(dataList);
    }

    private static void statDateListSort(List<JSONObject> list) {
        Collections.sort(list, new Comparator<JSONObject>() {
            @Override
            public int compare(JSONObject o1, JSONObject o2) {
                SimpleDateFormat format = new SimpleDateFormat("yyyy-MM-dd");
                try {
                    Date dt1 = format.parse(o1.getString("statDate"));
                    Date dt2 = format.parse(o2.getString("statDate"));
                    return dt1.compareTo(dt2);
                } catch (Exception e) {
                    e.printStackTrace();
                }
                return 0;
            }
        });
    }

    private static void statHourListSort(List<JSONObject> list) {
        Collections.sort(list, new Comparator<JSONObject>() {
            @Override
            public int compare(JSONObject o1, JSONObject o2) {
                try {
                    Integer dt1 = o1.getInteger("statHour");
                    Integer dt2 = o2.getInteger("statHour");
                    return dt1.compareTo(dt2);
                } catch (Exception e) {
                    e.printStackTrace();
                }
                return 0;
            }
        });
    }

    @Override
    public Result<Object> getMaxChargeAccountIdByUser(String userId) {
        JSONObject obj = etlDailyAccountMapper.getMaxChargeAccountId(userId);
        if (Check.isNull(obj)) {
            obj = etlDailyAccountMapper.getMaxChargeAccountId(null);
        }
        return Result.ok(obj.getLong("accountId"));
    }

    @Override
    public Result<Object> getMaxChargeAccountId(Long accountId) {
        JSONObject obj = new JSONObject();
        UserAllocation userAllocation = allocationMapper.getUserAllocation(accountId);
        if (!Check.isNull(userAllocation)) {
            obj.put("accountName", userAllocation.getAuthName());
            obj.put("status", userAllocation.getAccountStatus());
            obj.put("statusStr", userAllocation.getAccountStatus() == 0 ? "投放中" : "无效");
        }
        Integer counts = kuaiShouCreativeMapper.getRejectedCountsforAccountId(accountId);
        obj.put("rejectedCounts", counts);
        obj.put("accountId", accountId);
        CtopOauthToken oauthToken = oauthTokenService.getTokenByAccountId(accountId);
        if (Check.isNull(oauthToken)) {
            Result result = new Result();
            result.setResult(obj);
            result.setSuccess(false);
            result.setCode(500);
            result.setMessage("未获取到账户信息");
            return result;
        }
        JSONObject fundJson = iKuaishouInterfaceService.fundGet(oauthToken);
        if (!Check.isNull(fundJson)) {
            obj.put("totalBalance", fundJson.getBigDecimal("balance")); // 余额
        }
        return Result.ok(obj);
    }
}
